@@ -1,4 +1,3 @@
-
 @doc raw"""
     check_inverse_retraction(
         M::AbstractManifold,
@@ -42,24 +41,28 @@ no plot is generated,
 * `error`:             specify how to report errors: `:none`, `:info`, `:warn`, or `:error` are available
 * `window`:            specify window sizes within the `log_range` that are used for the slope estimation.
   the default is, to use all window sizes `2:N`.
+
+Note that since the plot yields more information than throwing an error, when both are specified,
+the plot is generated first and returned (to be shown/displayed), such that no error is thrown.
+You can switch to e.g. `:warn` to get a warning together with the plot.
 """
 function check_inverse_retraction(
-    M::AbstractManifold,
-    inverse_retraction_method::AbstractInverseRetractionMethod,
-    p = rand(M),
-    X = rand(M; vector_at = p);
-    exactness_tol::Real = 1e-12,
-    io::Union{IO,Nothing} = nothing,
-    limits = (-8.0, 0.0),
-    N::Int = 101,
-    second_order::Bool = true,
-    name::String = second_order ? "second order inverse retraction" : "inverse retraction",
-    log_range::AbstractVector = range(limits[1], limits[2]; length = N),
-    plot::Bool = false,
-    slope_tol::Real = 0.1,
-    error::Symbol = :none,
-    window = nothing,
-)
+        M::AbstractManifold,
+        inverse_retraction_method::AbstractInverseRetractionMethod,
+        p = rand(M),
+        X = rand(M; vector_at = p);
+        exactness_tol::Real = 1.0e-12,
+        io::Union{IO, Nothing} = nothing,
+        limits = (-8.0, 0.0),
+        N::Int = 101,
+        second_order::Bool = true,
+        name::String = second_order ? "second order inverse retraction" : "inverse retraction",
+        log_range::AbstractVector = range(limits[1], limits[2]; length = N),
+        plot::Bool = false,
+        slope_tol::Real = 0.1,
+        error::Symbol = :none,
+        window = nothing,
+    )
     Xn = X ./ norm(M, p, X) # normalize tangent direction
     # function for the directional derivative
     #
@@ -83,6 +86,103 @@ function check_inverse_retraction(
     )
 end
 
+function plot_check_geodesic end
+
+@doc raw"""
+    check_geodesic(
+        M::AbstractManifold,
+        p=rand(M),
+        X=rand(M; vector_at=p);
+        #
+        tol::Real = 1e-12,
+        io::Union{IO,Nothing} = nothing,
+        N::Int = 101,
+        name::String = "geodesic",
+        plot::Bool = false,
+        error::Symbol = :none,
+        inverse_retraction_method::AbstractInverseRetractionMethod = LogarithmicInverseRetraction(),
+        vector_transport_method::AbstractVectorTransportMethod = ParallelTransport(),
+    )
+
+Numerically check whether the [`geodesic`](@ref) implementation is correct.
+This check requires both the [`log`](@ref) and [`parallel_transport_to`](@ref) functions to be implemented
+in order to an exact numerical verification.
+We further require the [`norm`](@ref) function to be implemented for the [`AbstractManifold`](@ref) `M`.
+
+You can provide an `inverse_retraction_method` and `vector_transport_method` to be used,
+but the verification will then be less accurate.
+
+The tests performed are the following based on sampling the geodesic ``\gamma(t) = ``[`geodesic`](@ref)`(M, p, X)` at
+`N` equidistant points ``t_i = i/(N-1)``, `i=0,...,N-1` we denote by ``p_i = \gamma(t_i)``
+
+1. Compute the norms of the tangent vectors ``\dot\gamma(t_i) ≈ X_i = \log_{p_i}(p_{i+1})``, ``i=0,...,N-2``
+   and check how far these norms deviate from their mean value.
+2. Check how far this mean deviates from being ``\frac{||X||}{N-1}``
+3. Check that the parallel transport of each ``X_i`` from ``p_i`` to ``p_{i+1}`` is close to ``X_{i+1}``.
+
+# Arguments
+* `M`:    the manifold to check
+* `p`:    point on the manifold to start the geodesic, a random point is used by default
+* `X`:    tangent vector at `p` to start the geodesic, a random tangent vector is used by default
+
+# Keyword arguments
+* `tol`:     if all errors are below this tolerance, the geodesic is considered to be exact
+* `io`:      provide an `IO` to print the result to
+* `N`:       number of points to sample the geodesic on ``[0,1]`` (default: 101)
+* `plot`:    whether to plot the result (if `Plots.jl` is loaded).
+* `error`:   specify how to report errors: `:none`, `:info`, `:warn`, or `:error` are available
+* `inverse_retraction_method`:  method to use for the inverse retraction, it is recommended to use [`LogarithmicInverseRetraction`](@ref)
+* `vector_transport_method`:    method to use for the vector transport, it is recommended to use [`ParallelTransport`](@ref)
+
+Note that since the plot yields more information than throwing an error, when both are specified,
+the plot is generated first and returned (to be shown/displayed), such that no error is thrown.
+You can switch to e.g. `:warn` to get a warning together with the plot.
+"""
+function check_geodesic(
+        M::AbstractManifold,
+        p = rand(M),
+        X = rand(M; vector_at = p);
+        tol::Real = 1.0e-12,
+        io::Union{IO, Nothing} = nothing,
+        N::Int = 101,
+        plot::Bool = false,
+        error::Symbol = :none,
+        inverse_retraction_method::AbstractInverseRetractionMethod = LogarithmicInverseRetraction(),
+        vector_transport_method::AbstractVectorTransportMethod = ParallelTransport(),
+    )
+    T = range(0.0, 1.0; length = N)
+    γ = geodesic(M, p, X)
+    # points `p_i` to evaluate the error function at
+    ps = [ γ(t) for t in T ]
+    Xs = [ inverse_retract(M, ps[i], ps[i + 1], inverse_retraction_method) for i in 1:(N - 1) ]
+    norms = [ norm(M, p, X) for (p, X) in zip(ps[1:(end - 1)], Xs) ]
+    mean_norm = sum(norms) / length(norms)
+    # errors_1 how far awary are we from constant speed
+    errors_norm = [ abs(n - mean_norm) for n in norms ]
+    err_n_max = maximum(errors_norm)
+    # PT Xs to their neighbors, now Xi and Yi are in the same tangent space
+    Ys = [ vector_transport_to(M, ps[i + 1], Xs[i + 1], ps[i], vector_transport_method) for i in 1:(N - 2)]
+    # transporting (an approximation of) γ'(t_{i+1}) to T_{p_i}M should approximately equal γ'(t_i)
+    # (a) we measure this in norm
+    errors_pt = [ norm(M, p, X - Y) for (p, X, Y) in zip(ps[1:(N - 2)], Xs[1:(N - 2)], Ys) ]
+    err_pt_max = maximum(errors_pt)
+    # (b) we measure the angle preservation
+    errors_α = [ abs(1 - inner(M, p, X, Y) / (norm(M, p, X) * norm(M, p, Y))) for (p, X, Y) in zip(ps[1:(N - 2)], Xs[1:(N - 2)], Ys) ]
+    err_α_max = maximum(errors_α)
+    msg = """
+    Geodesic check results:
+    - max deviation from constant speed: $(err_n_max)
+    - max deviation from parallel transport: $(err_pt_max)
+    - max deviation from angle preservation: $(err_α_max)
+    """
+    (io !== nothing) && print(io, msg)
+    dre = (err_n_max > tol) || (err_pt_max > tol) || (err_α_max > tol)
+    dre && (error === :info) && @info msg
+    dre && (error === :warn) && @warn msg
+    plot && return ManifoldsBase.plot_check_geodesic(T, N, errors_norm, errors_pt, errors_α)
+    dre && (error === :error) && throw(ErrorException(msg))
+    return !dre
+end
 @doc raw"""
     check_retraction(
         M::AbstractManifold,
@@ -103,7 +203,7 @@ end
         window = nothing,
     )
 
-Check numerically wether the retraction `vector_transport_to` is correct, by selecting
+Check numerically whether the retraction `vector_transport_to` is correct, by selecting
 a set of points ``q_i = \exp_p (t_i X)`` where ``t`` takes all values from `log_range`,
 to then compare [`parallel_transport_to`](@ref) to the `vector_transport_method`
 applied to the vector `Y`.
@@ -131,24 +231,28 @@ no plot is generated,
 * `error`:             specify how to report errors: `:none`, `:info`, `:warn`, or `:error` are available
 * `window`:            specify window sizes within the `log_range` that are used for the slope estimation.
   the default is, to use all window sizes `2:N`.
+
+Note that since the plot yields more information than throwing an error, when both are specified,
+the plot is generated first and returned (to be shown/displayed), such that no error is thrown.
+You can switch to e.g. `:warn` to get a warning together with the plot.
 """
 function check_retraction(
-    M::AbstractManifold,
-    retraction_method::AbstractRetractionMethod,
-    p = rand(M),
-    X = rand(M; vector_at = p);
-    exactness_tol::Real = 1e-12,
-    io::Union{IO,Nothing} = nothing,
-    limits::Tuple = (-8.0, 0.0),
-    N::Int = 101,
-    second_order::Bool = true,
-    name::String = second_order ? "second order retraction" : "retraction",
-    log_range = range(limits[1], limits[2]; length = N),
-    plot::Bool = false,
-    slope_tol::Real = 0.1,
-    error::Symbol = :none,
-    window = nothing,
-)
+        M::AbstractManifold,
+        retraction_method::AbstractRetractionMethod,
+        p = rand(M),
+        X = rand(M; vector_at = p);
+        exactness_tol::Real = 1.0e-12,
+        io::Union{IO, Nothing} = nothing,
+        limits::Tuple = (-8.0, 0.0),
+        N::Int = 101,
+        second_order::Bool = true,
+        name::String = second_order ? "second order retraction" : "retraction",
+        log_range = range(limits[1], limits[2]; length = N),
+        plot::Bool = false,
+        slope_tol::Real = 0.1,
+        error::Symbol = :none,
+        window = nothing,
+    )
     Xn = X ./ norm(M, p, X) # normalize tangent direction
     # function for the directional derivative
     #
@@ -220,25 +324,29 @@ no plot is generated,
 * `error`:             specify how to report errors: `:none`, `:info`, `:warn`, or `:error` are available
 * `window`:            specify window sizes within the `log_range` that are used for the slope estimation.
   the default is, to use all window sizes `2:N`.
+
+Note that since the plot yields more information than throwing an error, when both are specified,
+the plot is generated first and returned (to be shown/displayed), such that no error is thrown.
+You can switch to e.g. `:warn` to get a warning together with the plot.
 """
 function check_vector_transport(
-    M::AbstractManifold,
-    vector_transport_method::AbstractVectorTransportMethod,
-    p = rand(M),
-    X = rand(M; vector_at = p),
-    Y = rand(M; vector_at = p);
-    exactness_tol::Real = 1e-12,
-    io::Union{IO,Nothing} = nothing,
-    limits::Tuple = (-8.0, 0.0),
-    N::Int = 101,
-    second_order::Bool = true,
-    name::String = second_order ? "second order vector transport" : "vector transport",
-    log_range::AbstractVector = range(limits[1], limits[2]; length = N),
-    plot::Bool = false,
-    slope_tol::Real = 0.1,
-    error::Symbol = :none,
-    window = nothing,
-)
+        M::AbstractManifold,
+        vector_transport_method::AbstractVectorTransportMethod,
+        p = rand(M),
+        X = rand(M; vector_at = p),
+        Y = rand(M; vector_at = p);
+        exactness_tol::Real = 1.0e-12,
+        io::Union{IO, Nothing} = nothing,
+        limits::Tuple = (-8.0, 0.0),
+        N::Int = 101,
+        second_order::Bool = true,
+        name::String = second_order ? "second order vector transport" : "vector transport",
+        log_range::AbstractVector = range(limits[1], limits[2]; length = N),
+        plot::Bool = false,
+        slope_tol::Real = 0.1,
+        error::Symbol = :none,
+        window = nothing,
+    )
     Xn = X ./ norm(M, p, X) # normalize tangent direction
     # function for the directional derivative
     #
@@ -313,20 +421,24 @@ no plot is be generated,
   The plot is in log-log-scale. This is returned and can then also be saved.
 * `slope_tol`:     tolerance for the slope (global) of the approximation
 * `error`:         specify how to handle errors, `:none`, `:info`, `:warn`, `:error`
+
+Note that since the plot yields more information than throwing an error, when both are specified,
+the plot is generated first and returned (to be shown/displayed), such that no error is thrown.
+You you can switch to e.g. `:warn` to get a warning together with the plot.
 """
 
 function prepare_check_result(
-    log_range::AbstractVector,
-    errors::AbstractVector,
-    slope::Real;
-    io::Union{IO,Nothing} = nothing,
-    name::String = "estimated slope",
-    slope_tol::Real = 1e-1,
-    plot::Bool = false,
-    error::Symbol = :none,
-    window = nothing,
-    exactness_tol::Real = 1e3 * eps(eltype(errors)),
-)
+        log_range::AbstractVector,
+        errors::AbstractVector,
+        slope::Real;
+        io::Union{IO, Nothing} = nothing,
+        name::String = "estimated slope",
+        slope_tol::Real = 1.0e-1,
+        plot::Bool = false,
+        error::Symbol = :none,
+        window = nothing,
+        exactness_tol::Real = 1.0e3 * eps(eltype(errors)),
+    )
     if max(errors...) < exactness_tol
         (io !== nothing) && print(
             io,
@@ -360,6 +472,8 @@ function prepare_check_result(
     (ab, bb, ib, jb) = find_best_slope_window(x, y, window; slope_tol = slope_tol)
     msg = "The $(name) fits best on [$(T[ib]),$(T[jb])] with slope  $(@sprintf("%.4f", bb)), but globally your slope $(@sprintf("%.4f", b)) is outside of the tolerance $slope ± $(slope_tol).\n"
     (io !== nothing) && print(io, msg)
+    (error === :info) && @info msg
+    (error === :warn) && @warn msg
     plot && return plot_slope(
         T,
         errors[errors .> 0];
@@ -370,8 +484,6 @@ function prepare_check_result(
         i = ib,
         j = jb,
     )
-    (error === :info) && @info msg
-    (error === :warn) && @warn msg
     (error === :error) && throw(ErrorException(msg))
     return false
 end
