@@ -27,6 +27,27 @@ An abstract type for representing approximate retraction methods.
 """
 abstract type ApproximateRetraction <: AbstractRetractionMethod end
 
+
+"""
+    ApproximateExponentialRetraction{N <: NamedTuple} <: ApproximateRetraction
+
+Retraction using an algorithm with certain parameters to approximate the exponential map.
+
+The intended implementation should use [`solve_chart_exp_ode`](@extref `Manifolds.solve_chart_exp_ode`).
+
+# Fields
+
+* `keyword_arguments` – a named tuple containing the parameters meant to be the keywords
+  for the approximation algorithm
+
+# Constructor
+
+    ApproximateExponentialRetraction(keyword_arguments::NamedTuple)
+"""
+struct ApproximateExponentialRetraction{N <: NamedTuple} <: ApproximateRetraction
+    keyword_arguments::N
+end
+
 @doc raw"""
     EmbeddedRetraction{T<:AbstractRetractionMethod} <: AbstractRetractionMethod
 
@@ -38,7 +59,7 @@ Compute a retraction by using the retraction of type `T` in the embedding and pr
 
 Generate the retraction with retraction `r` to use in the embedding.
 """
-struct EmbeddedRetraction{T<:AbstractRetractionMethod} <: AbstractRetractionMethod
+struct EmbeddedRetraction{T <: AbstractRetractionMethod} <: AbstractRetractionMethod
     retraction::T
 end
 
@@ -49,64 +70,6 @@ Retraction using the exponential map.
 """
 struct ExponentialRetraction <: AbstractRetractionMethod end
 
-
-@doc raw"""
-    ODEExponentialRetraction{T<:AbstractRetractionMethod, B<:AbstractBasis} <: AbstractRetractionMethod
-
-Approximate the exponential map on the manifold by evaluating the ODE descripting the geodesic at 1,
-assuming the default connection of the given manifold by solving the ordinary differential
-equation
-
-```math
-\frac{d^2}{dt^2} p^k + Γ^k_{ij} \frac{d}{dt} p_i \frac{d}{dt} p_j = 0,
-```
-
-where ``Γ^k_{ij}`` are the Christoffel symbols of the second kind, and
-the Einstein summation convention is assumed.
-
-# Constructor
-
-    ODEExponentialRetraction(
-        r::AbstractRetractionMethod,
-        b::AbstractBasis=DefaultOrthogonalBasis(),
-    )
-
-Generate the retraction with a retraction to use internally (for some approaches)
-and a basis for the tangent space(s).
-"""
-struct ODEExponentialRetraction{T<:AbstractRetractionMethod,B<:AbstractBasis} <:
-       AbstractRetractionMethod
-    retraction::T
-    basis::B
-end
-function ODEExponentialRetraction(r::T) where {T<:AbstractRetractionMethod}
-    return ODEExponentialRetraction(r, DefaultOrthonormalBasis())
-end
-function ODEExponentialRetraction(::T, b::CachedBasis) where {T<:AbstractRetractionMethod}
-    return throw(
-        DomainError(
-            b,
-            "Cached Bases are currently not supported, since the basis has to be implemented in a surrounding of the start point as well.",
-        ),
-    )
-end
-function ODEExponentialRetraction(r::ExponentialRetraction, ::AbstractBasis)
-    return throw(
-        DomainError(
-            r,
-            "You can not use the exponential map as an inner method to solve the ode for the exponential map.",
-        ),
-    )
-end
-function ODEExponentialRetraction(r::ExponentialRetraction, ::CachedBasis)
-    return throw(
-        DomainError(
-            r,
-            "Neither the exponential map nor a Cached Basis can be used with this retraction type.",
-        ),
-    )
-end
-
 """
     PolarRetraction <: AbstractRetractionMethod
 
@@ -164,12 +127,12 @@ Note that you can nest this type. Then the most outer specification of a keyword
 
 Specify the subtype `T <: `[`AbstractRetractionMethod`](@ref) to have keywords `kwargs...`.
 """
-struct RetractionWithKeywords{T<:AbstractRetractionMethod,K} <: AbstractRetractionMethod
+struct RetractionWithKeywords{T <: AbstractRetractionMethod, K} <: AbstractRetractionMethod
     retraction::T
     kwargs::K
 end
-function RetractionWithKeywords(m::T; kwargs...) where {T<:AbstractRetractionMethod}
-    return RetractionWithKeywords{T,typeof(kwargs)}(m, kwargs)
+function RetractionWithKeywords(m::T; kwargs...) where {T <: AbstractRetractionMethod}
+    return RetractionWithKeywords{T, typeof(kwargs)}(m, kwargs)
 end
 
 @doc raw"""
@@ -216,8 +179,11 @@ for numerical stability.
 # Constructor
 
     StabilizedRetraction(::AbstractRetractionMethod=ExponentialRetraction())
+
+# See also
+[`ExponentialRetraction`](@ref), [`embed_project!`](@ref)
 """
-struct StabilizedRetraction{TRM<:AbstractRetractionMethod} <: AbstractRetractionMethod
+struct StabilizedRetraction{TRM <: AbstractRetractionMethod} <: AbstractRetractionMethod
     retraction::TRM
 end
 StabilizedRetraction() = StabilizedRetraction(ExponentialRetraction())
@@ -250,11 +216,32 @@ A retraction based on the Cayley transform, which is realized by using the
 
 !!! note "Technical Note"
     Though you would call e.g. [`retract`](@ref)`(M, p, X, CayleyRetraction())`,
-    to implement a caley retraction, define [`retract_cayley!`](@ref)`(M, q, p, X, t)` for your manifold `M`.
+    to implement a Cayley retraction, define [`retract_cayley!`](@ref)`(M, q, p, X, t)` for your manifold `M`.
     By default both these functions fall back to calling a [`PadeRetraction`](@ref)`(1)`.
 """
 const CayleyRetraction = PadeRetraction{1}
 
+"""
+    ApproximateLogarithmicInverseRetraction{N <: NamedTuple} <: ApproximateInverseRetraction
+
+Inverse retraction using an algorithm with certain parameters to approximate the logarithmic map.
+
+The intended implementation should use [`solve_chart_log_bvp`](@extref `Manifolds.solve_chart_log_bvp`).
+
+# Fields
+
+* `keyword_arguments` – a named tuple containing the parameters meant to be the keywords
+  for the approximation algorithm
+
+# Constructor
+
+    ApproximateLogarithmicInverseRetraction(keyword_arguments::NamedTuple)
+"""
+struct ApproximateLogarithmicInverseRetraction{N <: NamedTuple} <: ApproximateInverseRetraction
+    keyword_arguments::N
+end
+
+
 @doc raw"""
    EmbeddedInverseRetraction{T<:AbstractInverseRetractionMethod} <: AbstractInverseRetractionMethod
 
@@ -266,8 +253,8 @@ Compute an inverse retraction by using the inverse retraction of type `T` in the
 
 Generate the inverse retraction with inverse retraction `r` to use in the embedding.
 """
-struct EmbeddedInverseRetraction{T<:AbstractInverseRetractionMethod} <:
-       AbstractInverseRetractionMethod
+struct EmbeddedInverseRetraction{T <: AbstractInverseRetractionMethod} <:
+    AbstractInverseRetractionMethod
     inverse_retraction::T
 end
 
@@ -297,6 +284,7 @@ function PadeInverseRetraction(m::Int)
     )
     return PadeInverseRetraction{m}()
 end
+
 @doc raw"""
     CayleyInverseRetraction <: AbstractInverseRetractionMethod
 
@@ -305,7 +293,7 @@ A retraction based on the Cayley transform, which is realized by using the
 
 !!! note "Technical Note"
     Though you would call e.g. [`inverse_retract`](@ref)`(M, p, q, CayleyInverseRetraction())`,
-    to implement an inverse caley retraction, define [`inverse_retract_cayley!`](@ref)`(M, X, p, q)` for your manifold `M`.
+    to implement an inverse Cayley retraction, define [`inverse_retract_cayley!`](@ref)`(M, X, p, q)` for your manifold `M`.
     By default both these functions fall back to calling a [`PadeInverseRetraction`](@ref)`(1)`.
 """
 const CayleyInverseRetraction = PadeInverseRetraction{1}
@@ -366,30 +354,22 @@ vector is projected before the retraction using `project`. If `project_point` is
 then the resulting point is projected after the retraction. `nlsolve_kwargs` are keyword
 arguments passed to `NLsolve.nlsolve`.
 """
-struct NLSolveInverseRetraction{TR<:AbstractRetractionMethod,TV,TK} <:
-       ApproximateInverseRetraction
+struct NLSolveInverseRetraction{TR <: AbstractRetractionMethod, TV, TK} <:
+    ApproximateInverseRetraction
     retraction::TR
     X0::TV
     project_tangent::Bool
     project_point::Bool
     nlsolve_kwargs::TK
     function NLSolveInverseRetraction(m, X0, project_point, project_tangent, nlsolve_kwargs)
-        return new{typeof(m),typeof(X0),typeof(nlsolve_kwargs)}(
-            m,
-            X0,
-            project_point,
-            project_tangent,
-            nlsolve_kwargs,
+        return new{typeof(m), typeof(X0), typeof(nlsolve_kwargs)}(
+            m, X0, project_point, project_tangent, nlsolve_kwargs,
         )
     end
 end
 function NLSolveInverseRetraction(
-    m,
-    X0 = nothing;
-    project_tangent::Bool = false,
-    project_point::Bool = false,
-    nlsolve_kwargs...,
-)
+        m, X0 = nothing; project_tangent::Bool = false, project_point::Bool = false, nlsolve_kwargs...,
+    )
     return NLSolveInverseRetraction(m, X0, project_point, project_tangent, nlsolve_kwargs)
 end
 
@@ -415,16 +395,16 @@ Note that you can nest this type. Then the most outer specification of a keyword
 
 Specify the subtype `T <: `[`AbstractInverseRetractionMethod`](@ref) to have keywords `kwargs...`.
 """
-struct InverseRetractionWithKeywords{T<:AbstractInverseRetractionMethod,K} <:
-       AbstractInverseRetractionMethod
+struct InverseRetractionWithKeywords{T <: AbstractInverseRetractionMethod, K} <:
+    AbstractInverseRetractionMethod
     inverse_retraction::T
     kwargs::K
 end
 function InverseRetractionWithKeywords(
-    m::T;
-    kwargs...,
-) where {T<:AbstractInverseRetractionMethod}
-    return InverseRetractionWithKeywords{T,typeof(kwargs)}(m, kwargs)
+        m::T;
+        kwargs...,
+    ) where {T <: AbstractInverseRetractionMethod}
+    return InverseRetractionWithKeywords{T, typeof(kwargs)}(m, kwargs)
 end
 
 """
@@ -438,6 +418,24 @@ Describes an inverse retraction that is based on the softmax function.
 """
 struct SoftmaxInverseRetraction <: AbstractInverseRetractionMethod end
 
+"""
+    StabilizedInveerseRetraction <: AbstractInverseRetractionMethod
+
+An inverse retraction wraps another inverse retraction and projects
+the resulting tangent vector onto the corresponding tangent space for numerical stability.
+
+# Constructor
+
+    StabilizedInverseRetraction(::AbstractInverseRetractionMethod=LogarithmicInverseRetraction())
+
+# See also
+[`LogarithmicInverseRetraction`](@ref), [`embed_project!`](@ref)
+"""
+struct StabilizedInverseRetraction{TRM <: AbstractInverseRetractionMethod} <: AbstractInverseRetractionMethod
+    inverse_retraction::TRM
+end
+StabilizedInverseRetraction() = StabilizedInverseRetraction(LogarithmicInverseRetraction())
+
 """
     default_inverse_retraction_method(M::AbstractManifold)
     default_inverse_retraction_method(M::AbstractManifold, ::Type{T}) where {T}
@@ -486,11 +484,9 @@ corresponding manifold.
 See also [`retract`](@ref).
 """
 function inverse_retract(
-    M::AbstractManifold,
-    p,
-    q,
-    m::AbstractInverseRetractionMethod = default_inverse_retraction_method(M, typeof(p)),
-)
+        M::AbstractManifold, p, q,
+        m::AbstractInverseRetractionMethod = default_inverse_retraction_method(M, typeof(p)),
+    )
     return _inverse_retract(M, p, q, m)
 end
 function _inverse_retract(M::AbstractManifold, p, q, ::LogarithmicInverseRetraction)
@@ -515,110 +511,82 @@ available methods.
 See also [`retract!`](@ref).
 """
 function inverse_retract!(
-    M::AbstractManifold,
-    X,
-    p,
-    q,
-    m::AbstractInverseRetractionMethod = default_inverse_retraction_method(M, typeof(p)),
-)
+        M::AbstractManifold, X, p, q,
+        m::AbstractInverseRetractionMethod = default_inverse_retraction_method(M, typeof(p)),
+    )
     return _inverse_retract!(M, X, p, q, m)
 end
 function _inverse_retract!(
-    M::AbstractManifold,
-    X,
-    p,
-    q,
-    ::LogarithmicInverseRetraction;
-    kwargs...,
-)
+        M::AbstractManifold, X, p, q, ::LogarithmicInverseRetraction; kwargs...,
+    )
     return log!(M, X, p, q; kwargs...)
 end
 
 #
 # dispatch to lower level
 function _inverse_retract!(
-    M::AbstractManifold,
-    X,
-    p,
-    q,
-    ::CayleyInverseRetraction;
-    kwargs...,
-)
+        M::AbstractManifold, X, p, q, m::ApproximateLogarithmicInverseRetraction; kwargs...,
+    )
+    return inverse_retract_approx!(M, X, p, q, m; kwargs...)
+end
+function _inverse_retract!(
+        M::AbstractManifold, X, p, q, ::CayleyInverseRetraction; kwargs...,
+    )
     return inverse_retract_cayley!(M, X, p, q; kwargs...)
 end
 function _inverse_retract!(
-    M::AbstractManifold,
-    X,
-    p,
-    q,
-    m::EmbeddedInverseRetraction;
-    kwargs...,
-)
+        M::AbstractManifold, X, p, q, m::EmbeddedInverseRetraction; kwargs...,
+    )
     return inverse_retract_embedded!(M, X, p, q, m.inverse_retraction; kwargs...)
 end
 function _inverse_retract!(
-    M::AbstractManifold,
-    X,
-    p,
-    q,
-    m::NLSolveInverseRetraction;
-    kwargs...,
-)
+        M::AbstractManifold, X, p, q, m::NLSolveInverseRetraction; kwargs...,
+    )
     return inverse_retract_nlsolve!(M, X, p, q, m; kwargs...)
 end
 function _inverse_retract!(
-    M::AbstractManifold,
-    X,
-    p,
-    q,
-    m::PadeInverseRetraction;
-    kwargs...,
-)
+        M::AbstractManifold, X, p, q, m::PadeInverseRetraction; kwargs...,
+    )
     return inverse_retract_pade!(M, X, p, q, m; kwargs...)
 end
 function _inverse_retract!(
-    M::AbstractManifold,
-    X,
-    p,
-    q,
-    ::PolarInverseRetraction;
-    kwargs...,
-)
+        M::AbstractManifold, X, p, q, ::PolarInverseRetraction; kwargs...,
+    )
     return inverse_retract_polar!(M, X, p, q; kwargs...)
 end
 function _inverse_retract!(
-    M::AbstractManifold,
-    X,
-    p,
-    q,
-    ::ProjectionInverseRetraction;
-    kwargs...,
-)
+        M::AbstractManifold, X, p, q, ::ProjectionInverseRetraction; kwargs...,
+    )
     return inverse_retract_project!(M, X, p, q; kwargs...)
 end
 function _inverse_retract!(M::AbstractManifold, X, p, q, ::QRInverseRetraction; kwargs...)
     return inverse_retract_qr!(M, X, p, q; kwargs...)
 end
 function _inverse_retract!(
-    M::AbstractManifold,
-    X,
-    p,
-    q,
-    m::InverseRetractionWithKeywords;
-    kwargs...,
-)
+        M::AbstractManifold, X, p, q, m::InverseRetractionWithKeywords; kwargs...,
+    )
     return _inverse_retract!(M, X, p, q, m.inverse_retraction; kwargs..., m.kwargs...)
 end
 function _inverse_retract!(
-    M::AbstractManifold,
-    X,
-    p,
-    q,
-    ::SoftmaxInverseRetraction;
-    kwargs...,
-)
+        M::AbstractManifold, X, p, q, ::SoftmaxInverseRetraction; kwargs...,
+    )
     return inverse_retract_softmax!(M, X, p, q; kwargs...)
 end
+function _inverse_retract!(
+        M::AbstractManifold, X, p, q, m::StabilizedInverseRetraction; kwargs...,
+    )
+    return inverse_retract_stabilized!(M, X, p, q, m; kwargs...)
+end
+
+"""
+    inverse_retract_approx!(M::AbstractManifold, p, q, m::ApproximateLogarithmicInverseRetraction)
+
+Compute the in-place variant of the [`ApproximateLogarithmicInverseRetraction`](@ref) `m`.
+"""
+inverse_retract_approx!(M::AbstractManifold, p, q, m::ApproximateLogarithmicInverseRetraction)
+
+function inverse_retract_approx! end
+
 """
     inverse_retract_embedded!(M::AbstractManifold, X, p, q, m::AbstractInverseRetractionMethod)
 
@@ -627,21 +595,12 @@ the [`AbstractInverseRetractionMethod`](@ref) `m` in the embedding (see [`get_em
 and projecting the result back.
 """
 function inverse_retract_embedded!(
-    M::AbstractManifold,
-    X,
-    p,
-    q,
-    m::AbstractInverseRetractionMethod,
-)
+        M::AbstractManifold, X, p::P, q, m::AbstractInverseRetractionMethod,
+    ) where {P}
     return project!(
-        M,
-        X,
-        p,
+        M, X, p,
         inverse_retract(
-            get_embedding(M),
-            embed(get_embedding(M), p),
-            embed(get_embedding(M), q),
-            m,
+            get_embedding(M, P), embed(get_embedding(M, P), p), embed(get_embedding(M, P), q), m,
         ),
     )
 end
@@ -710,6 +669,12 @@ inverse_retract_softmax!(M::AbstractManifold, X, p, q)
 
 function inverse_retract_softmax! end
 
+function inverse_retract_stabilized!(M::AbstractManifold, X, p, q, m::StabilizedInverseRetraction; kwargs...)
+    inverse_retract!(M, X, p, q, m.inverse_retraction; kwargs...)
+    embed_project!(M, X, p, X)
+    return X
+end
+
 _doc_retract = raw"""
     retract(M::AbstractManifold, p, X, method::AbstractRetractionMethod=default_retraction_method(M, typeof(p)))
     retract!(M::AbstractManifold, q, p, X, method::AbstractRetractionMethod=default_retraction_method(M, typeof(p)))
@@ -737,34 +702,34 @@ Locally, the retraction is invertible. For the inverse operation, see [`inverse_
 
 @doc "$(_doc_retract)"
 function retract(
-    M::AbstractManifold,
-    p,
-    X,
-    m::AbstractRetractionMethod = default_retraction_method(M, typeof(p)),
-)
-    return _retract(M, p, X, m)
+        M::AbstractManifold, p, X,
+        m::AbstractRetractionMethod = default_retraction_method(M, typeof(p));
+        kwargs...
+    )
+    return _retract(M, p, X, m; kwargs...)
 end
 # Layer 2 – dispatch on the method
-function _retract(M::AbstractManifold, p, X, ::ExponentialRetraction)
-    return exp(M, p, X)
+function _retract(M::AbstractManifold, p, X, ::ExponentialRetraction; kwargs...)
+    return exp(M, p, X; kwargs...)
 end
-function _retract(M::AbstractManifold, p, X, m::AbstractRetractionMethod)
+function _retract(M::AbstractManifold, p, X, m::AbstractRetractionMethod; kwargs...)
     q = allocate_result(M, retract, p, X)
-    return retract!(M, q, p, X, m)
+    return retract!(M, q, p, X, m; kwargs...)
 end
 
 @doc "$(_doc_retract)"
 function retract!(
-    M::AbstractManifold,
-    q,
-    p,
-    X,
-    method::AbstractRetractionMethod = default_retraction_method(M, typeof(p)),
-)
-    return _retract!(M, q, p, X, method)
+        M::AbstractManifold, q, p, X,
+        method::AbstractRetractionMethod = default_retraction_method(M, typeof(p));
+        kwargs...
+    )
+    return _retract!(M, q, p, X, method; kwargs...)
 end
 
 # Retract Layer 2 – dispatch on the method
+function _retract!(M::AbstractManifold, q, p, X, m::ApproximateExponentialRetraction; kwargs...)
+    return retract_approx!(M, q, p, X, m; kwargs...)
+end
 function _retract!(M::AbstractManifold, q, p, X, ::CayleyRetraction; kwargs...)
     return retract_cayley!(M, q, p, X; kwargs...)
 end
@@ -774,9 +739,6 @@ end
 function _retract!(M::AbstractManifold, q, p, X, ::ExponentialRetraction; kwargs...)
     return exp!(M, q, p, X; kwargs...)
 end
-function _retract!(M::AbstractManifold, q, p, X, m::ODEExponentialRetraction; kwargs...)
-    return retract_exp_ode!(M, q, p, X, m.retraction, m.basis; kwargs...)
-end
 function _retract!(M::AbstractManifold, q, p, X, ::PolarRetraction; kwargs...)
     return retract_polar!(M, q, p, X; kwargs...)
 end
@@ -826,12 +788,9 @@ By default, this falls back to calling [`retract`](@ref) with `t*X`.
 
 @doc "$(_doc_retract_fused)"
 function retract_fused(
-    M::AbstractManifold,
-    p,
-    X,
-    t::Number,
-    m::AbstractRetractionMethod = default_retraction_method(M, typeof(p)),
-)
+        M::AbstractManifold, p, X, t::Number,
+        m::AbstractRetractionMethod = default_retraction_method(M, typeof(p)),
+    )
     return _retract_fused(M, p, X, t, m)
 end
 
@@ -845,81 +804,42 @@ end
 
 @doc "$(_doc_retract_fused)"
 function retract_fused!(
-    M::AbstractManifold,
-    q,
-    p,
-    X,
-    t::Number,
-    m::AbstractRetractionMethod = default_retraction_method(M, typeof(p)),
-)
-    return _retract_fused!(M, q, p, X, t, m)
+        M::AbstractManifold, q, p, X, t::Number,
+        m::AbstractRetractionMethod = default_retraction_method(M, typeof(p));
+        kwargs...
+    )
+    return _retract_fused!(M, q, p, X, t, m; kwargs...)
 end
 # Retract fused Layer 2
 # Retract Layer 2 – dispatch on the method
 function _retract_fused!(
-    M::AbstractManifold,
-    q,
-    p,
-    X,
-    t::Number,
-    ::CayleyRetraction;
-    kwargs...,
-)
+        M::AbstractManifold, q, p, X, t::Number, m::ApproximateExponentialRetraction; kwargs...,
+    )
+    return retract_approx_fused!(M, q, p, X, t, m; kwargs...)
+end
+function _retract_fused!(
+        M::AbstractManifold, q, p, X, t::Number, ::CayleyRetraction; kwargs...,
+    )
     return retract_cayley_fused!(M, q, p, X, t; kwargs...)
 end
 function _retract_fused!(
-    M::AbstractManifold,
-    q,
-    p,
-    X,
-    t::Number,
-    m::EmbeddedRetraction;
-    kwargs...,
-)
+        M::AbstractManifold, q, p, X, t::Number, m::EmbeddedRetraction; kwargs...,
+    )
     return retract_embedded_fused!(M, q, p, X, t, m.retraction; kwargs...)
 end
 function _retract_fused!(
-    M::AbstractManifold,
-    q,
-    p,
-    X,
-    t::Number,
-    ::ExponentialRetraction;
-    kwargs...,
-)
+        M::AbstractManifold, q, p, X, t::Number, ::ExponentialRetraction; kwargs...,
+    )
     return exp_fused!(M, q, p, X, t; kwargs...)
 end
 function _retract_fused!(
-    M::AbstractManifold,
-    q,
-    p,
-    X,
-    t::Number,
-    m::ODEExponentialRetraction;
-    kwargs...,
-)
-    return retract_exp_ode_fused!(M, q, p, X, t, m.retraction, m.basis; kwargs...)
-end
-function _retract_fused!(
-    M::AbstractManifold,
-    q,
-    p,
-    X,
-    t::Number,
-    ::PolarRetraction;
-    kwargs...,
-)
+        M::AbstractManifold, q, p, X, t::Number, ::PolarRetraction; kwargs...,
+    )
     return retract_polar_fused!(M, q, p, X, t; kwargs...)
 end
 function _retract_fused!(
-    M::AbstractManifold,
-    q,
-    p,
-    X,
-    t::Number,
-    ::ProjectionRetraction;
-    kwargs...,
-)
+        M::AbstractManifold, q, p, X, t::Number, ::ProjectionRetraction; kwargs...,
+    )
     return retract_project_fused!(M, q, p, X, t; kwargs...)
 end
 function _retract_fused!(M::AbstractManifold, q, p, X, t::Number, ::QRRetraction; kwargs...)
@@ -929,39 +849,21 @@ function _retract_fused!(M::AbstractManifold, q, p, X, t::Number, m::SasakiRetra
     return retract_sasaki_fused!(M, q, p, X, t, m)
 end
 function _retract_fused!(
-    M::AbstractManifold,
-    q,
-    p,
-    X,
-    t::Number,
-    ::SoftmaxRetraction;
-    kwargs...,
-)
+        M::AbstractManifold, q, p, X, t::Number, ::SoftmaxRetraction; kwargs...,
+    )
     return retract_softmax_fused!(M, q, p, X, t; kwargs...)
 end
-function _retract_fused!(M::AbstractManifold, q, p, X, t::Number, m::StabilizedRetraction)
-    return retract_stabilized_fused!(M, q, p, X, t, m)
+function _retract_fused!(M::AbstractManifold, q, p, X, t::Number, m::StabilizedRetraction; kwargs...)
+    return retract_stabilized_fused!(M, q, p, X, t, m; kwargs...)
 end
 function _retract_fused!(
-    M::AbstractManifold,
-    q,
-    p,
-    X,
-    t::Number,
-    m::PadeRetraction;
-    kwargs...,
-)
+        M::AbstractManifold, q, p, X, t::Number, m::PadeRetraction; kwargs...,
+    )
     return retract_pade_fused!(M, q, p, X, t, m; kwargs...)
 end
 function _retract_fused!(
-    M::AbstractManifold,
-    q,
-    p,
-    X,
-    t::Number,
-    m::RetractionWithKeywords;
-    kwargs...,
-)
+        M::AbstractManifold, q, p, X, t::Number, m::RetractionWithKeywords; kwargs...,
+    )
     return _retract_fused!(M, q, p, X, t, m.retraction; kwargs..., m.kwargs...)
 end
 
@@ -969,6 +871,16 @@ end
 #
 # retract and retract_fused layer 3
 
+function retract_approx! end
+"""
+    retract_approx!(M::AbstractManifold, q, p, X)
+
+Compute the in-place variant of the [`ApproximateExponentialRetraction`](@ref) `m`.
+"""
+retract_approx!(M::AbstractManifold, q, p, X, m::ApproximateExponentialRetraction)
+
+function retract_approx_fused! end
+
 """
     retract_embedded!(M::AbstractManifold, q, p, X, m::AbstractRetractionMethod)
 
@@ -977,21 +889,13 @@ the [`AbstractRetractionMethod`](@ref) `m` in the embedding (see [`get_embedding
 and projecting the result back.
 """
 function retract_embedded!(
-    M::AbstractManifold,
-    q,
-    p,
-    X,
-    m::AbstractRetractionMethod;
-    kwargs...,
-)
+        M::AbstractManifold, q, p::P, X, m::AbstractRetractionMethod; kwargs...,
+    ) where {P}
     return project!(
         M,
         q,
         retract(
-            get_embedding(M),
-            embed(get_embedding(M), p),
-            embed(get_embedding(M), p, X),
-            m;
+            get_embedding(M, P), embed(get_embedding(M, P), p), embed(get_embedding(M, P), p, X), m;
             kwargs...,
         ),
     )
@@ -1003,24 +907,13 @@ end
 Compute the scaled variant of `retract_embedded!`.
 """
 function retract_embedded_fused!(
-    M::AbstractManifold,
-    q,
-    p,
-    X,
-    t::Number,
-    m::AbstractRetractionMethod;
-    kwargs...,
-)
+        M::AbstractManifold, q, p::P, X, t::Number, m::AbstractRetractionMethod; kwargs...,
+    ) where {P}
     return project!(
         M,
         q,
         retract_fused(
-            get_embedding(M),
-            embed(get_embedding(M), p),
-            embed(get_embedding(M), p, X),
-            t,
-            m;
-            kwargs...,
+            get_embedding(M, P), embed(get_embedding(M, P), p), embed(get_embedding(M, P), p, X), t, m; kwargs...,
         ),
     )
 end
@@ -1039,25 +932,6 @@ function retract_cayley_fused!(M::AbstractManifold, q, p, X, t::Number; kwargs..
     return retract_cayley!(M, q, p, t * X; kwargs...)
 end
 
-function retract_exp_ode! end
-"""
-    retract_exp_ode!(M::AbstractManifold, q, p, X, m::AbstractRetractionMethod, B::AbstractBasis)
-
-Compute the in-place variant of the [`ODEExponentialRetraction`](@ref)`(m, B)`.
-"""
-retract_exp_ode!(
-    M::AbstractManifold,
-    q,
-    p,
-    X,
-    m::AbstractRetractionMethod,
-    B::AbstractBasis,
-)
-
-function retract_exp_ode_fused!(M::AbstractManifold, q, p, X, t::Number, m, B)
-    return retract_exp_ode!(M, q, p, t * X, m, B)
-end
-
 function retract_pade! end
 """
     retract_pade!(M::AbstractManifold, q, p, X, m::PadeRetraction)
@@ -1143,19 +1017,22 @@ function retract_stabilized!(M::AbstractManifold, q, p, X, m::StabilizedRetracti
 end
 
 function retract_stabilized_fused!(
-    M::AbstractManifold,
-    q,
-    p,
-    X,
-    t::Number,
-    m::StabilizedRetraction,
-)
+        M::AbstractManifold, q, p, X, t::Number, m::StabilizedRetraction,
+    )
     retract_fused!(M, q, p, X, t, m.retraction)
     return embed_project!(M, q, q)
 end
 
 Base.show(io::IO, ::CayleyRetraction) = print(io, "CayleyRetraction()")
 Base.show(io::IO, ::PadeRetraction{m}) where {m} = print(io, "PadeRetraction($m)")
+function Base.show(io::IO, s::StabilizedRetraction)
+    r = s.retraction isa ExponentialRetraction ? "" : "$(s.retraction)"
+    return print(io, "StabilizedRetraction($r)")
+end
+function Base.show(io::IO, s::StabilizedInverseRetraction)
+    r = s.inverse_retraction isa LogarithmicInverseRetraction ? "" : "$(s.inverse_retraction)"
+    return print(io, "StabilizedInverseRetraction($r)")
+end
 
 #
 # default estimation methods pass down with and without the point type
