@@ -21,34 +21,20 @@ using ManifoldsBaseTestUtils
 
         # ProjectionRetraction only works <= 1 well in stepsize
         @test_throws ErrorException check_retraction(
-            M,
-            ProjectionRetraction(),
-            p,
-            X;
-            limits = (-2.5, 2.0),
-            error = :error,
+            M, ProjectionRetraction(), p, X; limits = (-2.5, 2.0), error = :error,
         )
         @test !check_retraction(M, ProjectionRetraction(), p, X; limits = (-2.5, 2.0))
 
         #test window size error
         @test_throws ErrorException ManifoldsBase.find_best_slope_window(
-            zeros(2),
-            zeros(2),
-            20,
+            zeros(2), zeros(2), 20,
         )
         @test_throws ErrorException ManifoldsBase.find_best_slope_window(
-            zeros(2),
-            zeros(2),
-            [2, 20],
+            zeros(2), zeros(2), [2, 20],
         )
-        @test check_retraction(M, ExponentialRetraction(), p, X; exactness_tol = 1e-7)
+        @test check_retraction(M, ExponentialRetraction(), p, X; exactness_tol = 1.0e-7)
         check_retraction(
-            M,
-            ExponentialRetraction(),
-            p,
-            X;
-            plot = true,
-            exactness_tol = 1e-7,
+            M, ExponentialRetraction(), p, X; plot = true, exactness_tol = 1.0e-7,
         )
     end
     @testset "Test inverse_retract checks" begin
@@ -98,7 +84,7 @@ using ManifoldsBaseTestUtils
             LogarithmicInverseRetraction(),
             p,
             X;
-            exactness_tol = 1e-7,
+            exactness_tol = 1.0e-7,
         )
         check_inverse_retraction(
             M,
@@ -106,7 +92,7 @@ using ManifoldsBaseTestUtils
             p,
             X;
             plot = true,
-            exactness_tol = 1e-7,
+            exactness_tol = 1.0e-7,
         )
     end
     @testset "Test vector_transport_to checks" begin
@@ -158,7 +144,7 @@ using ManifoldsBaseTestUtils
             limits = (-2.5, 2.0),
         )
         # Check exactness case
-        @test check_vector_transport(M, ParallelTransport(), p, X, Y; exactness_tol = 1e-7)
+        @test check_vector_transport(M, ParallelTransport(), p, X, Y; exactness_tol = 1.0e-7)
         check_vector_transport(
             M,
             ParallelTransport(),
@@ -166,10 +152,10 @@ using ManifoldsBaseTestUtils
             X,
             Y;
             plot = true,
-            exactness_tol = 1e-7,
+            exactness_tol = 1.0e-7,
         )
     end
-    @testset "StabilizedRetraction" begin
+    @testset "StabilizedRetraction and its inverse" begin
         M = TestSphere(10)
         p1 = [
             -0.3676232664793671,
@@ -205,44 +191,78 @@ using ManifoldsBaseTestUtils
         X3 = copy(X1)
         X4 = copy(X1)
         X5 = copy(X1)
+        # inverse
+        p6 = copy(p1) # keep for checks
+        p7 = exp(M, p1, X1)
+        p8 = exp(M, p1, X1)
+        p9 = exp(M, p1, X1)
+        Y1 = copy(X1)
+        Y2 = copy(X1)
+        Y3 = copy(X1)
         q = similar(p1)
         SR = StabilizedRetraction()
-        err_eps = 1e-5
+        @test repr(SR) == "StabilizedRetraction()"
+        SIR = StabilizedInverseRetraction()
+        @test repr(SIR) == "StabilizedInverseRetraction()"
+        err_eps = 1.0e-5
         for _ in 1:1000
-
             X1 .+= err_eps
             exp!(M, q, p1, X1)
             parallel_transport_to!(M, X1, p1, X1, q)
             p1 .= q
-
             # mutating non-fused
             X2 .+= err_eps
             retract!(M, q, p2, X2, SR)
             parallel_transport_to!(M, X2, p2, X2, q)
             p2 .= q
-
             # mutating fused
             X3 .+= err_eps
             ManifoldsBase.retract_fused!(M, q, p3, X3, 1.0, SR)
             parallel_transport_to!(M, X3, p3, X3, q)
             p3 .= q
-
             # non-mutating non-fused
             X4 .+= err_eps
             q4 = retract(M, p4, X4, SR)
             parallel_transport_to!(M, X4, p4, X4, q4)
             p4 .= q4
-
             # non-mutating fused
             X5 .+= err_eps
             q5 = ManifoldsBase.retract_fused(M, p5, X5, 1.0, SR)
             parallel_transport_to!(M, X5, p5, X5, q5)
             p5 .= q5
+
+            p7 .+= err_eps
+            Y1 .= log(M, p6, p7)
+            p8 .+= err_eps
+            Y2 .= inverse_retract(M, p6, p8, SIR)
+            p9 .+= err_eps
+            inverse_retract!(M, Y3, p6, p9, SIR)
         end
         @test !is_point(M, p1)
-        @test is_point(M, p2)
-        @test is_point(M, p3)
-        @test is_point(M, p4)
-        @test is_point(M, p5)
+        @test is_point(M, p2; error = :error)
+        @test is_point(M, p3; error = :error)
+        @test is_point(M, p4; error = :error)
+        @test is_point(M, p5; error = :error)
+        # test the inverse as well
+        @test !is_vector(M, p6, Y1)
+        @test is_vector(M, p6, Y2; error = :error, atol = 1.0e-16)
+        @test is_vector(M, p6, Y3; error = :error, atol = 1.0e-16)
+    end
+    @testset "Test check_geodesic" begin
+        M = TestSphere(10)
+        q = zeros(11)
+        q[1] = 1.0
+        p = zeros(11)
+        p[1:4] .= 1 / sqrt(4)
+        X = log(M, p, q)
+        @test check_geodesic(M, p, X)
+        # One call with generating a plot
+        check_geodesic(M, p, X; plot = true)
+
+        # Check with too large steps
+        X2 = zeros(11) # but make it non-tangent to q
+        X2[1] = 1.0
+        X2[2:4] .= 2.0
+        @test_throws ErrorException check_geodesic(M, q, X2; error = :error)
     end
 end
