@@ -20,27 +20,34 @@ power_array_wrapper(::Type{NestedPowerRepresentation}, ::Int) = identity
 power_array_wrapper(::Type{NestedReplacingPowerRepresentation}, i::Int) = SVector{i}
 
 function ManifoldsBase.allocate(
-    ::ManifoldsBase.PowerManifoldNestedReplacing,
-    x::AbstractArray{<:SArray},
-)
+        ::ManifoldsBase.PowerManifoldNestedReplacing,
+        x::AbstractArray{<:SArray},
+    )
     return similar(x)
 end
 
 function ManifoldsBase.representation_size(
-    M::ManifoldsBaseTestUtils.TestPowerManifoldMultidimensional,
-)
+        M::ManifoldsBaseTestUtils.TestPowerManifoldMultidimensional,
+    )
     return (representation_size(M.manifold)..., ManifoldsBase.get_parameter(M.size)...)
 end
 
 @inline function ManifoldsBase._write(
-    ::ManifoldsBaseTestUtils.TestPowerManifoldMultidimensional,
-    rep_size::Tuple,
-    x::AbstractArray,
-    i::Tuple,
-)
+        ::ManifoldsBaseTestUtils.TestPowerManifoldMultidimensional,
+        rep_size::Tuple,
+        x::AbstractArray,
+        i::Tuple,
+    )
     return view(x, ManifoldsBase.rep_size_to_colons(rep_size)..., i...)
 end
 
+function ManifoldsBase.get_embedding(
+        M::PowerManifold{𝔽, TM, TSW, ManifoldsBaseTestUtils.TestArrayRepresentation},
+        P::Type,
+    ) where {𝔽, TM <: AbstractManifold{𝔽}, TSW}
+    ME = get_embedding(M.manifold, P)
+    return PowerManifold{ManifoldsBase._get_field(ME), typeof(ME), TSW, ManifoldsBaseTestUtils.TestArrayRepresentation}(ME, M.size)
+end
 
 @testset "Power Manifold" begin
 
@@ -49,7 +56,7 @@ end
         N = PowerManifold(M, ManifoldsBaseTestUtils.TestArrayRepresentation(), 2)
         O = PowerManifold(N, ManifoldsBaseTestUtils.TestArrayRepresentation(), 3) # joins instead of nesting.
         @test repr(O) ==
-              "PowerManifold(DefaultManifold(3; field = ℝ), ManifoldsBaseTestUtils.TestArrayRepresentation(), 2, 3)"
+            "PowerManifold(DefaultManifold(3; field = ℝ), ManifoldsBaseTestUtils.TestArrayRepresentation(), 2, 3)"
         p = zeros(6)
         X = zeros(6)
         @test has_components(N)
@@ -61,6 +68,7 @@ end
         @test default_inverse_retraction_method(M) == default_inverse_retraction_method(N)
         @test default_retraction_method(M) == default_retraction_method(N)
         @test default_vector_transport_method(M) == default_vector_transport_method(N)
+        @test get_embedding(N, typeof(p)) === N
     end
 
     @testset "PowerManifold and allocation with empty representation size" begin
@@ -71,6 +79,7 @@ end
         # check - though only because this function exists for avoiding ambiguities.
         cm = ManifoldsBase.allocate_result(N, get_coordinates, p, X, DefaultBasis())
         @test size(X) == size(cm)
+        @test project(N, p) == p
     end
 
     @testset "PowerManifoldNested with mutable element" begin
@@ -79,13 +88,17 @@ end
         p = [UpperTriangular([1 2; 2 1]), UpperTriangular([1 2; 2 1])]
         q = [UpperTriangular([2 3; 3 2]), UpperTriangular([1 2; 2 1])]
         @test typeof(log(N, p, q)) === typeof(p)
+        # DefaultManifold is always its own embedding independent of the point type, so
+        @test get_embedding(N) == get_embedding(N, typeof(p))
     end
 
     @testset "PowerManifoldNestedReplacing with SArray element" begin
         M = ManifoldsBase.DefaultManifold(2, 2)
         N = PowerManifold(M, NestedReplacingPowerRepresentation(), 2)
-        p = [SMatrix{2,2,Float64}([i i+1; i-1 i-2]) for i in 1:2]
-        allocate(M, p) isa Vector{SMatrix{2,2,Float64,4}}
+        p = [SMatrix{2, 2, Float64}([i i + 1; i - 1 i - 2]) for i in 1:2]
+        allocate(M, p) isa Vector{SMatrix{2, 2, Float64, 4}}
+        # DefaultManifold is always its own embedding independent of the point type, so
+        @test get_embedding(N) == get_embedding(N, typeof(p))
     end
 
     @testset "allocate_on" begin
@@ -106,6 +119,8 @@ end
         X = allocate_on(N, TangentSpaceType(), Vector{Matrix{Float32}})
         @test X isa Vector{Matrix{Float32}}
         @test size(X) == (2,)
+
+        @test project(N, p) isa typeof(p)
     end
 
     for PowerRepr in [NestedPowerRepresentation, NestedReplacingPowerRepresentation]
@@ -124,16 +139,16 @@ end
 
                 @testset "Constructors" begin
                     @test repr(N) ==
-                          "PowerManifold(DefaultManifold(3; field = ℝ), $(PowerRepr)(), $pow_string)"
+                        "PowerManifold(DefaultManifold(3; field = ℝ), $(PowerRepr)(), $pow_string)"
                     # add to type
                     @test repr(N^3) ==
-                          "PowerManifold(DefaultManifold(3; field = ℝ), $(PowerRepr)(), $pow_string, 3)"
+                        "PowerManifold(DefaultManifold(3; field = ℝ), $(PowerRepr)(), $pow_string, 3)"
                     # add to type
                     @test repr(PowerManifold(N, 3)) ==
-                          "PowerManifold(DefaultManifold(3; field = ℝ), $(PowerRepr)(), $pow_string, 3)"
+                        "PowerManifold(DefaultManifold(3; field = ℝ), $(PowerRepr)(), $pow_string, 3)"
                     # nest
                     @test repr(PowerManifold(N, PowerRepr(), 3)) ==
-                          "PowerManifold(PowerManifold(DefaultManifold(3; field = ℝ), $(PowerRepr)(), $pow_string), $(PowerRepr)(), 3)"
+                        "PowerManifold(PowerManifold(DefaultManifold(3; field = ℝ), $(PowerRepr)(), $pow_string), $(PowerRepr)(), 3)"
                 end
 
                 @test N^3 == PowerManifold(M, PowerRepr(), pow_size..., 3)
@@ -141,7 +156,7 @@ end
                 if pow_size == (2,)
 
                     @test ManifoldsBase.get_iterator(N^3) ==
-                          Base.product(Base.OneTo(2), Base.OneTo(3))
+                        Base.product(Base.OneTo(2), Base.OneTo(3))
                     @testset "point/tangent checks" begin
                         pE1 = [wrapper_3(zeros(3)), wrapper_2(ones(2))] # one component wrong
                         pE2 = wrapper_2.([zeros(2), ones(2)]) # both wrong
@@ -220,11 +235,11 @@ end
                         @test ManifoldsBase.get_iterator(N) == Base.OneTo(2)
                     else
                         @test ManifoldsBase.get_iterator(N) ==
-                              Base.product(Base.OneTo(2), Base.OneTo(3))
+                            Base.product(Base.OneTo(2), Base.OneTo(3))
                     end
                     @test injectivity_radius(N) == injectivity_radius(M)
                     @test injectivity_radius(N, ExponentialRetraction()) ==
-                          injectivity_radius(M)
+                        injectivity_radius(M)
                     @test injectivity_radius(N, p) == injectivity_radius(M, p)
                     @test is_flat(N)
                     p2 = allocate(p)
@@ -303,7 +318,7 @@ end
                     # the method tested below should not be used but it prevents ambiguities from occurring
                     # and the test is here to make coverage happy
                     @test ManifoldsBase.allocate_result(N, get_coordinates, p, q, B) isa
-                          Vector
+                        Vector
                     v2 = similar(v)
                     get_coordinates!(N, v2, p, q, B)
                     @test v2 == v
@@ -326,12 +341,12 @@ end
                     B3 = get_basis(N, p, B2)
                     if pow_size == (2,)
                         @test sprint(show, "text/plain", B) ==
-                              """$(DefaultBasis()) for a power manifold
-          Basis for component (1,):
-          $(sprint(show, "text/plain", B.data.bases[1]))
-          Basis for component (2,):
-          $(sprint(show, "text/plain", B.data.bases[2]))
-          """
+                            """$(DefaultBasis()) for a power manifold
+                            Basis for component (1,):
+                            $(sprint(show, "text/plain", B.data.bases[1]))
+                            Basis for component (2,):
+                            $(sprint(show, "text/plain", B.data.bases[2]))
+                            """
                     end
 
                     bv = get_vectors(N, p, B)
@@ -369,15 +384,15 @@ end
             @test is_point(N, rand(MersenneTwister(123), N))
             @test rand(MersenneTwister(123), N) == rand(MersenneTwister(123), N)
             p = rand(N)
-            @test is_vector(N, p, rand(N; vector_at = p); atol = 1e-15)
+            @test is_vector(N, p, rand(N; vector_at = p); atol = 1.0e-15)
             @test is_vector(
                 N,
                 p,
                 rand(MersenneTwister(123), N; vector_at = p);
-                atol = 1e-15,
+                atol = 1.0e-15,
             )
             @test rand(MersenneTwister(123), N; vector_at = p) ==
-                  rand(MersenneTwister(123), N; vector_at = p)
+                rand(MersenneTwister(123), N; vector_at = p)
         end
     end
 
@@ -441,13 +456,13 @@ end
         @test PowerManifold(N, 3) isa PowerManifold{
             ℝ,
             <:DefaultManifold,
-            ManifoldsBase.TypeParameter{Tuple{2,3}},
+            ManifoldsBase.TypeParameter{Tuple{2, 3}},
             NestedPowerRepresentation,
         }
         @test PowerManifold(N, 3; parameter = :field) isa
-              PowerManifold{ℝ,<:DefaultManifold,Tuple{Int,Int},NestedPowerRepresentation}
+            PowerManifold{ℝ, <:DefaultManifold, Tuple{Int, Int}, NestedPowerRepresentation}
         @test repr(N) ==
-              "PowerManifold(DefaultManifold(3; field = ℝ), NestedPowerRepresentation(), 2; parameter=:type)"
+            "PowerManifold(DefaultManifold(3; field = ℝ), NestedPowerRepresentation(), 2; parameter=:type)"
 
         M = ManifoldsBase.DefaultManifold(3)
         N = PowerManifold(M, NestedPowerRepresentation(), 2, 3; parameter = :type)
