@@ -1,17 +1,19 @@
 using Test
 using ManifoldsBase
 @testset "ManifoldsBase" begin
-    bound = 6
+    bound = 8
     # six ambiguities come from possible incorrectly formed calls to `allocate`
-    num_ambiguities = length(Test.detect_ambiguities(ManifoldsBase))
+    ambiguities = Test.detect_ambiguities(ManifoldsBase)
+    num_ambiguities = length(ambiguities)
     #num_ambiguities > 0 && @warn "The number of ambiguities in ManifoldsBase is $(num_ambiguities)."
-    if VERSION >= v"1.10-DEV"
-        # One ambiguity from JSON library loaded by VSCode
-        @test num_ambiguities <= bound + 1
+    if num_ambiguities > bound
+        for amb in ambiguities
+            println(amb)
+        end
     end
+    @test num_ambiguities <= bound
 
     include("decorator_traits.jl")
-    include("passthrough_decorator.jl")
     include("allocation.jl")
     include("numbers.jl")
     include("bases.jl")
@@ -30,6 +32,7 @@ using ManifoldsBase
     include("metric.jl")
     include("fibers.jl")
     include("numerical_checks.jl")
+    include("quotient_manifold.jl")
     include("deprecated.jl")
     include("test_aqua.jl")
 end
